// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "util/DawnTool.hpp"
#include "util/File.hpp"
#include "util/XmlParser.hpp"
#include "util/Language.cpp"

namespace Dawn {
  struct LanguageGroup {
    std::vector<struct LanguageString> strings;
    std::string key;
    std::string lang;
  };

  struct LanguageRoot {
    std::string lang;
    std::vector<struct LanguageString> strings;
  };

  class LanguageParser : public XmlParser<struct LanguageString> {
    protected:
      std::vector<std::string> getRequiredAttributes();
      std::map<std::string, std::string> getOptionalAttributes();
      int32_t onParse(
        Xml *node,
        std::map<std::string, std::string> values,
        struct LanguageString *out,
        std::string *error
      );
  };

  class LanguageGroupParser : public XmlParser<struct LanguageGroup> {
    protected:
      std::vector<std::string> getRequiredAttributes();
      std::map<std::string, std::string> getOptionalAttributes();
      int32_t onParse(
        Xml *node,
        std::map<std::string, std::string> values,
        struct LanguageGroup *out,
        std::string *error
      );
  };
  
  class LanguageRootParser : public XmlParser<struct LanguageRoot> {
    protected:
      std::vector<std::string> getRequiredAttributes();
      std::map<std::string, std::string> getOptionalAttributes();
      int32_t onParse(
        Xml *node,
        std::map<std::string, std::string> values,
        struct LanguageRoot *out,
        std::string *error
      ) override;
  };

  class LanguageGen : public DawnTool {
    protected:
      std::vector<std::string> getRequiredFlags() override;
      
      int32_t parseGroup(
        Xml *node,
        std::string key,
        std::map<std::string, std::vector<struct LanguageString>> *strings
      );

      int32_t parseString(
        Xml *node,
        std::string key,
        std::map<std::string, std::vector<struct LanguageString>> *strings
      );

    public:
      int32_t start() override;
  };
}